package com.friday.tooldriver.mapper.taieoc;

import com.friday.base.common.response.DubboResult;
import com.friday.operation.api.keepalive.dto.smartrpa.*;
import com.friday.operation.api.keepalive.dto.taieoc.MachineLicenseFilterDto;
import com.friday.tooldriver.po.taieoc.MachineLicensePo;
import org.springframework.stereotype.Repository;

import java.util.List;

@Repository
public interface MachineLicenseMapper {
    void insert(MachineLicensePo po);

    List<MachineLicensePo> selectMachineLicense(MachineLicenseFilterDto dto);

    void deleteByPrimaryKey(Long id);


    int updateMachineLicense(MachineLicenseAuthUpdateDto req);

    int checkExistByRobotKey(String robotKey);

    int insertLicenseAll(MachineLicenseAuthUpdateDto req);

    int checkLicenseIsUsed(String license);

    MachineLicenseInfo selectMachineLicenseInfo(MachineLicenseInfoReq dto);
}
